import os
import time
import threading
import Queue


class _LightSwitch:

    def __init__(self, lock):
        self.__counter = 0
        self.__mutex = threading.Lock()
        self.__lock = lock

    def acquire(self):
        self.__mutex.acquire()
        self.__counter += 1
        if self.__counter == 1:
            self.__lock.acquire()
        self.__mutex.release()

    def release(self):
        self.__mutex.acquire()
        self.__counter -= 1
        if self.__counter == 0:
            self.__lock.release()
        self.__mutex.release()

    def __enter__(self):
        return self.acquire()

    def __exit__(self, exc_value, exc_type, traceback):
        self.release()


def RWLock():
    no_readers = threading.Lock()
    no_writers = threading.Lock()
    readers_queue = threading.Lock()
    read_switch = _LightSwitch(no_writers)
    write_switch = _LightSwitch(no_readers)

    class ReadLock:

        def acquire(self):
            readers_queue.acquire()
            no_readers.acquire()
            read_switch.acquire()
            no_readers.release()
            readers_queue.release()

        def release(self):
            read_switch.release()

        def __enter__(self):
            return self.acquire()

        def __exit__(self, exc_value, exc_type, traceback):
            self.release()

    class WriteLock:

        def acquire(self):
            write_switch.acquire()
            no_writers.acquire()

        def release(self):
            no_writers.release()
            write_switch.release()

        def __enter__(self):
            return self.acquire()

        def __exit__(self, exc_value, exc_type, traceback):
            self.release()

    return ReadLock(), WriteLock()


class Thread(threading.Thread):

    def __init__(self, target, *args):
        threading.Thread.__init__(self, target=target, args=args)
        self.daemon = True
        self.start()


ASK = object()
ALWAYS = object()
NEVER = object()
override = ASK


def ask_dialog(path):
    ask = None
    while ask not in ("yes", "no", "never", "always"):
        print "Should I override %s?" % path
        ask = raw_input("type 'yes', 'no', 'never' or 'always': ")
    return ask


def copy(path, t):
    print "copying", path
    time.sleep(t)
    print "finished copying", path


def worker(q, r, w, t):
    global override
    while True:
        path = q.get()
        if os.path.exists(path):
            with w:
                if override is ALWAYS:
                    pass
                elif override is NEVER:
                    q.task_done()
                    continue
                elif override is ASK:
                    o = ask_dialog(path)
                    if o in ("no", "never"):
                        if o == "never":
                            override = NEVER
                        q.task_done()
                        continue
                    elif o == "always":
                        override = ALWAYS
        with r:
            copy(path, t)
            q.task_done()


if __name__ == "__main__":
    r, w = RWLock()
    q = Queue.Queue()
    paths = ("/tmp/cucu", "/tmp/cucucu", "/etc/passwd", "/etc/group")
    for path in paths:
        q.put(path)
    [Thread(worker, q, r, w, t) for t in (1, 2, 1, 1)]
    q.join()
